/**************************************************************************
*  Copyright (c) 2013-2014 by Michael Fischer.
*  All rights reserved.
*
*  Based on an example from ST. Therefore partial copyright:
*  Copyright (c) 2011 STMicroelectronics

*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*  
*  1. Redistributions of source code must retain the above copyright 
*     notice, this list of conditions and the following disclaimer.
*  2. Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the 
*     documentation and/or other materials provided with the distribution.
*  3. Neither the name of the author nor the names of its contributors may 
*     be used to endorse or promote products derived from this software 
*     without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
*  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
*  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
*  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
*  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
*  SUCH DAMAGE.
*
***************************************************************************
*  History:
*
*  19.05.2013  mifi  First Version.
*  19.08.2013  mifi  Some rework and cleanup.
*  21.08.2013  mifi  Some changes for ChibiOS.
*  18.04.2014  mifi  low_level_init will init the HW by BoardETHConfig.
**************************************************************************/

/*
 * Copyright (c) 2001-2004 Swedish Institute of Computer Science.
 * All rights reserved. 
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED 
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
 * SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
 *
 * This file is part of the lwIP TCP/IP stack.
 * 
 * Author: Adam Dunkels <adam@sics.se>
 *
 */

#define __ETHERNETIF_C__

/*=======================================================================*/
/*  Include                                                              */
/*=======================================================================*/
#include <string.h>

#include "ethernetif.h"
#include "project.h"

#include "lwip\tcpip.h"
#include "netif\etharp.h"
#include "lwip\stats.h"

/* Forward declarations. */
static void ethernetif_input (void *arg);

/*=======================================================================*/
/*  Extern                                                               */
/*=======================================================================*/

/* Ethernet Rx & Tx DMA Descriptors */
extern ETH_DMADESCTypeDef  DMARxDscrTab[ETH_RXBUFNB], DMATxDscrTab[ETH_TXBUFNB];

/* Ethernet Transmit buffers */
extern uint8_t Tx_Buff[ETH_TXBUFNB][ETH_TX_BUF_SIZE]; 

/* Ethernet Receive buffers  */
extern uint8_t Rx_Buff[ETH_RXBUFNB][ETH_RX_BUF_SIZE]; 

/* Global pointer for last received frame infos */
extern ETH_DMA_Rx_Frame_infos *DMA_RX_FRAME_infos;

/* Global pointers to track current transmit and receive descriptors */
extern ETH_DMADESCTypeDef  *DMATxDescToSet;

/*=======================================================================*/
/*  All Structures and Common Constants                                  */
/*=======================================================================*/

#define IFNAME0 'e'
#define IFNAME1 'n'

#define NETIF_EVENT_RX        0x0001
#define NETIF_EVENT_LINK_UP   0x0002
#define NETIF_EVENT_LINK_DOWN 0x0004

#define NETIF_EVENT_WAIT_ALL  (NETIF_EVENT_RX        | \
                               NETIF_EVENT_LINK_UP   | \
                               NETIF_EVENT_LINK_DOWN )
                                                             
#define LINK_TMR_INTERVAL     1000   

/*=======================================================================*/
/*  Definition of all local Data                                         */
/*=======================================================================*/

static sys_event_t     NetIfEvent;
static sys_sem_t       TxSemaphore;
static IP_PHY_DRIVER *pPHY = NULL;

/*=======================================================================*/
/*  Definition of all local Procedures                                   */
/*=======================================================================*/

/**
 * Timer callback function for checking the Ethernet Link.
 *
 * @param arg contain the PHYDriver
 */
static void LinkTimer (void *arg)
{
   IP_PHY_DRIVER *pPHY           = (IP_PHY_DRIVER *)arg;
   static uint8_t  OldLinkStatus = PHY_LINK_STATUS_NO_LINK;
   uint8_t         LinkStatus;
   uint32_t        Value;
   
   /* Get actual Link status */
   LinkStatus = pPHY->GetLinkStatus(pPHY);

   /* Check if the status has changed */
   if (LinkStatus != OldLinkStatus)
   {
      /* Check for DOWN/UP */
      if (PHY_LINK_STATUS_NO_LINK == LinkStatus)
      {
         /* Send event */
         sys_event_signal(&NetIfEvent, NETIF_EVENT_LINK_DOWN);
      }
      else
      {
         /* Update MAC */
         Value  = ETH->MACCR;
         Value &= ~(ETH_MACCR_FES | ETH_MACCR_DM);
      
         if (LinkStatus & PHY_LINK_STATUS_SPEED_100M)
         {
            Value |= ETH_MACCR_FES;
         }
      
         if (LinkStatus & PHY_LINK_STATUS_MODE_FULL)
         {
            Value |= ETH_MACCR_DM;
         }
      
         ETH->MACCR = Value;

         /* Send event */
         sys_event_signal(&NetIfEvent, NETIF_EVENT_LINK_UP);
      }
   } /* end if (LinkStatus != OldLinkStatus) */
   
   OldLinkStatus = LinkStatus;      
   
   sys_timeout(LINK_TMR_INTERVAL, LinkTimer, pPHY);
} /* LinkTimer */


/**
 * In this function, the hardware should be initialized.
 * Called from ethernetif_init().
 *
 * @param netif the already initialized lwip network interface structure
 *        for this ethernetif
 */
static err_t low_level_init (struct netif *netif)
{
   netif->flags = 0;
   
   /* Configure the HW of the board, and return the PHY info */
   pPHY = BoardETHConfig();
   if (NULL == pPHY)
   {
      /* Error, no PHY available */
      return(ERR_MEM);
   }
   
   /* Set netif MAC hardware address length */
   netif->hwaddr_len = ETHARP_HWADDR_LEN;
   
   /* Set netif MAC hardware address */
   memcpy(netif->hwaddr, netif->state, ETHARP_HWADDR_LEN); 
  
   /* Set netif maximum transfer unit */
   netif->mtu = 1500;

   /* Accept broadcast address and ARP traffic */
   netif->flags = NETIF_FLAG_BROADCAST | NETIF_FLAG_ETHARP;
  
   /* Create semaphore used for low_level_output */
   sys_sem_new(&TxSemaphore, 1);

   /* Set the MAC address in the MAC */ 
   ETH_MACAddressConfig(ETH_MAC_Address0, netif->hwaddr); 
  
   /* Initialize Tx Descriptors list: Chain Mode */
   ETH_DMATxDescChainInit(DMATxDscrTab, &Tx_Buff[0][0], ETH_TXBUFNB);
   
   /* Initialize Rx Descriptors list: Chain Mode  */
   ETH_DMARxDescChainInit(DMARxDscrTab, &Rx_Buff[0][0], ETH_RXBUFNB);
  
   /* Enable Ethernet Rx interrrupt */
   for (int i=0; i<ETH_RXBUFNB; i++)
   {
      ETH_DMARxDescReceiveITConfig(&DMARxDscrTab[i], ENABLE);
   }

#ifdef CHECKSUM_BY_HARDWARE
   /* Enable the checksum insertion for the Tx frames */
   for (int i=0; i<ETH_TXBUFNB; i++)
   {
      ETH_DMATxDescChecksumInsertionConfig(&DMATxDscrTab[i], ETH_DMATxDesc_ChecksumTCPUDPICMPFull);
   }
#endif

   /* 
    * Create the IPRX thread 
    *
    * PHY init, ETH start and the LinkTimer will be 
    * handled by the ethernetif_input thread.
    */   
   sys_thread_new ("iprx_thread", ethernetif_input, netif,
                   TASK_IP_RX_STK_SIZE, TASK_IP_RX_PRIORITY);

   return(ERR_OK);
} /* low_level_init */


/**
 * This function should do the actual transmission of the packet. The packet is
 * contained in the pbuf that is passed to the function. This pbuf
 * might be chained.
 *
 * @param netif the lwip network interface structure for this ethernetif
 * @param p the MAC packet to send (e.g. IP packet including MAC addresses and type)
 * @return ERR_OK if the packet could be sent
 *         an err_t value if the packet couldn't be sent
 *
 * @note Returning ERR_MEM here if a DMA queue of your MAC is full can lead to
 *       strange results. You might consider waiting for space in the DMA queue
 *       to become availale since the stack doesn't retry to send a packet
 *       dropped because of memory failure (except for the TCP timers).
 */
static err_t low_level_output (struct netif *netif, struct pbuf *p)
{
   struct pbuf *q;
   u16 l = 0;
   u8 *buffer;
   
   (void)netif;
   
   sys_arch_sem_wait(&TxSemaphore, 0);
   
   buffer =  (u8 *)(DMATxDescToSet->Buffer1Addr);
   for(q = p; q != NULL; q = q->next) 
   {
      memcpy((u8_t*)&buffer[l], q->payload, q->len);
      l = l + q->len;
   }
   
   ETH_Prepare_Transmit_Descriptors(l);
   
   LINK_STATS_INC(link.xmit);
   
   sys_sem_signal(&TxSemaphore);

   return(ERR_OK);
} /* low_level_output */


/**
 * Should allocate a pbuf and transfer the bytes of the incoming
 * packet from the interface into the pbuf.
 *
 * @param netif the lwip network interface structure for this ethernetif
 * @return a pbuf filled with the received packet (including MAC header)
 *         NULL on memory error
 */
static struct pbuf *low_level_input (struct netif *netif)
{
   struct pbuf *p, *q;
   volatile u16_t len;
   uint32_t i=0;
   
   FrameTypeDef frame;
   u8          *buffer;
   __IO ETH_DMADESCTypeDef *DMARxNextDesc;
  
   (void)netif;

   p = NULL;
  
   /* Get received frame */
   frame = ETH_Get_Received_Frame_interrupt();
   
   /* Check if any packet received */
   if (0 == frame.length) return(NULL);
  
   /* Check that frame has no error */
   if ((frame.descriptor->Status & ETH_DMARxDesc_ES) == (uint32_t)RESET)
   {
      /* Obtain the size of the packet and put it into the "len" variable. */
      len = (u16)frame.length;
      buffer = (u8 *)frame.buffer;
      
      /* We allocate a pbuf chain of pbufs from the pool. */
      p = pbuf_alloc(PBUF_RAW, len, PBUF_POOL);
 
      /* Copy received frame from ethernet driver buffer to stack buffer */
      if (p != NULL)
      { 
         i = 0;
         for (q = p; q != NULL; q = q->next)
         {
            memcpy((u8_t*)q->payload, (u8_t*)&buffer[i], q->len);
            i = i + q->len;
         } 
         
         LINK_STATS_INC(link.recv);
      }
   }
  
   /* Release descriptors to DMA */
   /* Check if received frame with multiple DMA buffer segments */
   if (DMA_RX_FRAME_infos->Seg_Count > 1)
   {
      DMARxNextDesc = DMA_RX_FRAME_infos->FS_Rx_Desc;
   }
   else
   {
      DMARxNextDesc = frame.descriptor;
   }
  
   /* Set Own bit in Rx descriptors: gives the buffers back to DMA */
   for (i = 0; i < DMA_RX_FRAME_infos->Seg_Count; i++)
   {  
      DMARxNextDesc->Status = ETH_DMARxDesc_OWN;
      DMARxNextDesc = (ETH_DMADESCTypeDef *)(DMARxNextDesc->Buffer2NextDescAddr);
   }
  
   /* Clear Segment_Count */
   DMA_RX_FRAME_infos->Seg_Count =0;
  
   /* When Rx Buffer unavailable flag is set: clear it and resume reception */
   if ((ETH->DMASR & ETH_DMASR_RBUS) != (u32)RESET)  
   {
      /* Clear RBUS ETHERNET DMA flag */
      ETH->DMASR = ETH_DMASR_RBUS;
      
      /* Resume DMA reception */
      ETH->DMARPDR = 0;
   }
   
   return(p);
} /* low_level_input */


/**
 * This function should be called when a packet is ready to be read
 * from the interface. It uses the function low_level_input() that
 * should handle the actual reception of bytes from the network
 * interface. Then the type of the received packet is determined and
 * the appropriate input function is called.
 *
 * @param netif the lwip network interface structure for this ethernetif
 */
static void ethernetif_input (void *arg)
{
   struct pbuf    *p;
   struct netif   *netif = (struct netif*)arg;
   struct eth_hdr *ethhdr;
   unsigned        Event;
   
   /* Create the event used for RX and LinkStatus */
   sys_event_new(&NetIfEvent, NETIF_EVENT_WAIT_ALL);

   /* Init the PHY */
   pPHY->Init(pPHY);

   /* Now the MAC can be started */
   ETH_Start(); 

   /* Create LinkTimer to check the LinkStatus */   
   sys_timeout(LINK_TMR_INTERVAL, LinkTimer, pPHY);

   while (1)
   {
      /* Wait for an event */
      Event = sys_event_wait(&NetIfEvent, NETIF_EVENT_WAIT_ALL);
      if (Event & NETIF_EVENT_RX)
      {
         while (1)
         {
            /* move received packet into a new pbuf */
            p = low_level_input( netif );
            
            /* no packet could be read, silently ignore this */
            if (p == NULL) break;
            
            /* points to packet payload, which starts with an Ethernet header */
            ethhdr = p->payload;
            
            switch (htons(ethhdr->type)) 
            {
               /* IP or ARP packet? */
               case ETHTYPE_IP:
               case ETHTYPE_ARP:
#if PPPOE_SUPPORT
               /* PPPoE packet? */
               case ETHTYPE_PPPOEDISC:
               case ETHTYPE_PPPOE:
#endif /* PPPOE_SUPPORT */
               
                  /* full packet send to tcpip_thread to process */
                  if (netif->input(p, netif) != ERR_OK)
                  {
                     LWIP_DEBUGF(NETIF_DEBUG, ("ethernetif_input: IP input error\n"));
                     pbuf_free(p);
                  }
                  break;
               
               default:
                  pbuf_free(p);
                  break;                  
            } /* end switch (htons(ethhdr->type)) */               
         } /* while (1), loop over low_level_input */
      } /* if (Event & NETIF_EVENT_RX) */
      
      if(Event & NETIF_EVENT_LINK_UP)
      {
         tcpip_callback_with_block((tcpip_callback_fn)netif_set_link_up,
                                   netif, 0);
      }
      
      if(Event & NETIF_EVENT_LINK_DOWN)
      {
         tcpip_callback_with_block((tcpip_callback_fn)netif_set_link_down,
                                   netif, 0);
      }
      
   } /* end while (1), task loop */
} /* ethernetif_input */

/*=======================================================================*/
/*  All code exported                                                    */
/*=======================================================================*/

/**
 * Ethernet IRQ handler
 */
void ETH_IRQHandler (void)
{
   CH_IRQ_PROLOGUE();
   
   /* Frame received */
   if (ETH_GetDMAFlagStatus(ETH_DMA_FLAG_R) == SET) 
   {
      /* Send a event to wakeup LwIP task */
      sys_event_signal_i(&NetIfEvent, NETIF_EVENT_RX);
   }        
	
   /* Clear the interrupt flags. */
   /* Clear the Eth DMA Rx IT pending bits */
   ETH_DMAClearITPendingBit(ETH_DMA_IT_R);
   ETH_DMAClearITPendingBit(ETH_DMA_IT_NIS);
      
   CH_IRQ_EPILOGUE();
} /* ETH_IRQHandler */


/**
 * Should be called at the beginning of the program to set up the
 * network interface. It calls the function low_level_init() to do the
 * actual setup of the hardware.
 *
 * This function should be passed as a parameter to netif_add().
 *
 * @param netif the lwip network interface structure for this ethernetif
 * @return ERR_OK if the loopif is initialized
 *         ERR_MEM if private data couldn't be allocated
 *         any other err_t on error
 */
err_t ethernetif_init (struct netif *netif)
{
   err_t error = ERR_MEM;
   
   if (netif != NULL)
   {
#if LWIP_NETIF_HOSTNAME
      /* Initialize interface hostname */
      netif->hostname = "lwip";
#endif /* LWIP_NETIF_HOSTNAME */

      /* Descriptive abbreviation for this interface */
      netif->name[0] = IFNAME0;
      netif->name[1] = IFNAME1;
  
      /* 
       * We directly use etharp_output() here to save a function call.
       * You can instead declare your own function an call etharp_output()
       * from it if you have to do some checks before sending (e.g. if link
       * is available...) 
       */
      netif->output     = etharp_output;
      netif->linkoutput = low_level_output;
  
      /* Initialize the hardware */
      error = low_level_init(netif);
   }      

   return(error);
} /* ethernetif_init */

/*** EOF ***/
